/* automatically generated from appfinder-preferences.glade */
#ifdef __SUNPRO_C
#pragma align 4 (appfinder_preferences_ui)
#endif
#ifdef __GNUC__
static const char appfinder_preferences_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char appfinder_preferences_ui[] =
#endif
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?><interface><requires lib=\"gt"
  "k+\" version=\"2.20\"/><object class=\"GtkListStore\" id=\"action-types"
  "\"><columns><column type=\"gchararray\"/></columns><data><row><col id=\""
  "0\" translatable=\"yes\">Prefix</col></row><row><col id=\"0\" translata"
  "ble=\"yes\">Regular Expression</col></row></data></object><object class"
  "=\"GtkListStore\" id=\"actions-store\"><columns><column type=\"gchararr"
  "ay\"/><column type=\"gint\"/></columns></object><object class=\"GtkList"
  "Store\" id=\"icon-sizes\"><columns><column type=\"gchararray\"/></colum"
  "ns><data><row><col id=\"0\" translatable=\"yes\">Very Small</col></row>"
  "<row><col id=\"0\" translatable=\"yes\">Smaller</col></row><row><col id"
  "=\"0\" translatable=\"yes\">Small</col></row><row><col id=\"0\" transla"
  "table=\"yes\">Normal</col></row><row><col id=\"0\" translatable=\"yes\""
  ">Large</col></row><row><col id=\"0\" translatable=\"yes\">Larger</col><"
  "/row><row><col id=\"0\" translatable=\"yes\">Very Large</col></row></da"
  "ta></object><object class=\"GtkImage\" id=\"image3\"><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">False</property><p"
  "roperty name=\"stock\">gtk-clear</property></object><object class=\"Xfc"
  "eTitledDialog\" id=\"dialog\"><property name=\"can_focus\">False</prope"
  "rty><property name=\"title\" translatable=\"yes\">Application Finder</p"
  "roperty><property name=\"default_width\">385</property><property name=\""
  "default_height\">425</property><property name=\"icon_name\">gtk-prefere"
  "nces</property><property name=\"type_hint\">dialog</property><child int"
  "ernal-child=\"vbox\"><object class=\"GtkVBox\" id=\"dialog-vbox1\"><pro"
  "perty name=\"visible\">True</property><property name=\"can_focus\">Fals"
  "e</property><property name=\"spacing\">2</property><child internal-chil"
  "d=\"action_area\"><object class=\"GtkHButtonBox\" id=\"dialog-action_ar"
  "ea1\"><property name=\"visible\">True</property><property name=\"can_fo"
  "cus\">False</property><property name=\"layout_style\">end</property><ch"
  "ild><object class=\"GtkButton\" id=\"button-close\"><property name=\"la"
  "bel\">gtk-close</property><property name=\"use_action_appearance\">Fals"
  "e</property><property name=\"visible\">True</property><property name=\""
  "can_focus\">True</property><property name=\"receives_default\">True</pr"
  "operty><property name=\"use_stock\">True</property></object><packing><p"
  "roperty name=\"expand\">False</property><property name=\"fill\">False</"
  "property><property name=\"position\">0</property></packing></child><chi"
  "ld><object class=\"GtkButton\" id=\"button-help\"><property name=\"labe"
  "l\">gtk-help</property><property name=\"use_action_appearance\">False</"
  "property><property name=\"visible\">True</property><property name=\"can"
  "_focus\">True</property><property name=\"receives_default\">False</prop"
  "erty><property name=\"use_stock\">True</property></object><packing><pro"
  "perty name=\"expand\">False</property><property name=\"fill\">False</pr"
  "operty><property name=\"position\">0</property><property name=\"seconda"
  "ry\">True</property></packing></child></object><packing><property name="
  "\"expand\">False</property><property name=\"fill\">True</property><prop"
  "erty name=\"pack_type\">end</property><property name=\"position\">0</pr"
  "operty></packing></child><child><object class=\"GtkNotebook\" id=\"note"
  "book1\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">True</property><property name=\"border_width\">6</property><chi"
  "ld><object class=\"GtkVBox\" id=\"vbox3\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"border_width\">6</property><property name=\"spacing\">6</property><"
  "child><object class=\"GtkFrame\" id=\"frame1\"><property name=\"visible"
  "\">True</property><property name=\"can_focus\">False</property><propert"
  "y name=\"label_xalign\">0</property><property name=\"shadow_type\">none"
  "</property><child><object class=\"GtkAlignment\" id=\"alignment2\"><pro"
  "perty name=\"visible\">True</property><property name=\"can_focus\">Fals"
  "e</property><property name=\"left_padding\">12</property><child><object"
  " class=\"GtkVBox\" id=\"vbox4\"><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">False</property><property name=\"border"
  "_width\">6</property><property name=\"spacing\">6</property><child><obj"
  "ect class=\"GtkCheckButton\" id=\"remember-category\"><property name=\""
  "label\" translatable=\"yes\">Remember last _selected category</property"
  "><property name=\"use_action_appearance\">False</property><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">True</propert"
  "y><property name=\"receives_default\">False</property><property name=\""
  "use_underline\">True</property><property name=\"draw_indicator\">True</"
  "property></object><packing><property name=\"expand\">True</property><pr"
  "operty name=\"fill\">True</property><property name=\"position\">0</prop"
  "erty></packing></child><child><object class=\"GtkCheckButton\" id=\"alw"
  "ays-center\"><property name=\"label\" translatable=\"yes\">Always c_ent"
  "er the window</property><property name=\"use_action_appearance\">False<"
  "/property><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">True</property><property name=\"receives_default\">False</pro"
  "perty><property name=\"tooltip_text\" translatable=\"yes\">Center the w"
  "indow on startup.</property><property name=\"use_underline\">True</prop"
  "erty><property name=\"draw_indicator\">True</property></object><packing"
  "><property name=\"expand\">True</property><property name=\"fill\">True<"
  "/property><property name=\"position\">1</property></packing></child><ch"
  "ild><object class=\"GtkCheckButton\" id=\"enable-service\"><property na"
  "me=\"label\" translatable=\"yes\">Keep running _instance in the backgro"
  "und</property><property name=\"use_action_appearance\">False</property>"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "True</property><property name=\"receives_default\">False</property><pro"
  "perty name=\"tooltip_text\" translatable=\"yes\">Instead of quitting th"
  "e application when the last window is closed, keep a running instance t"
  "o speed up opening new windows. You might want to disable this to reduc"
  "e memory usage.</property><property name=\"use_underline\">True</proper"
  "ty><property name=\"draw_indicator\">True</property><property name=\"ac"
  "tive\">True</property></object><packing><property name=\"expand\">True<"
  "/property><property name=\"fill\">True</property><property name=\"posit"
  "ion\">2</property></packing></child></object></child></object></child><"
  "child type=\"label\"><object class=\"GtkLabel\" id=\"label6\"><property"
  " name=\"visible\">True</property><property name=\"can_focus\">False</pr"
  "operty><property name=\"label\" translatable=\"yes\">Behaviour</propert"
  "y><attributes><attribute name=\"weight\" value=\"bold\"/></attributes><"
  "/object></child></object><packing><property name=\"expand\">False</prop"
  "erty><property name=\"fill\">True</property><property name=\"position\""
  ">0</property></packing></child><child><object class=\"GtkFrame\" id=\"f"
  "rame3\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"label_xalign\">0</property><pr"
  "operty name=\"shadow_type\">none</property><child><object class=\"GtkAl"
  "ignment\" id=\"alignment5\"><property name=\"visible\">True</property><"
  "property name=\"can_focus\">False</property><property name=\"left_paddi"
  "ng\">12</property><child><object class=\"GtkTable\" id=\"table2\"><prop"
  "erty name=\"visible\">True</property><property name=\"can_focus\">False"
  "</property><property name=\"border_width\">6</property><property name=\""
  "n_rows\">4</property><property name=\"n_columns\">2</property><property"
  " name=\"column_spacing\">12</property><property name=\"row_spacing\">6<"
  "/property><child><object class=\"GtkCheckButton\" id=\"icon-view\"><pro"
  "perty name=\"label\" translatable=\"yes\">_View items as icons</propert"
  "y><property name=\"use_action_appearance\">False</property><property na"
  "me=\"visible\">True</property><property name=\"can_focus\">True</proper"
  "ty><property name=\"receives_default\">False</property><property name=\""
  "use_underline\">True</property><property name=\"draw_indicator\">True</"
  "property></object><packing><property name=\"right_attach\">2</property>"
  "</packing></child><child><object class=\"GtkCheckButton\" id=\"text-bes"
  "ide-icons\"><property name=\"label\" translatable=\"yes\">Text besi_de "
  "icons</property><property name=\"use_action_appearance\">False</propert"
  "y><property name=\"visible\">True</property><property name=\"can_focus\""
  ">True</property><property name=\"receives_default\">False</property><pr"
  "operty name=\"use_underline\">True</property><property name=\"draw_indi"
  "cator\">True</property></object><packing><property name=\"right_attach\""
  ">2</property><property name=\"top_attach\">1</property></packing></chil"
  "d><child><object class=\"GtkLabel\" id=\"label9\"><property name=\"visi"
  "ble\">True</property><property name=\"can_focus\">False</property><prop"
  "erty name=\"xalign\">0</property><property name=\"label\" translatable="
  "\"yes\">Ite_m icon size:</property><property name=\"use_underline\">Tru"
  "e</property><property name=\"mnemonic_widget\">item-icon-size</property"
  "></object><packing><property name=\"top_attach\">2</property><property "
  "name=\"bottom_attach\">3</property><property name=\"x_options\">GTK_FIL"
  "L</property></packing></child><child><object class=\"GtkLabel\" id=\"la"
  "bel10\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"xalign\">0</property><property"
  " name=\"label\" translatable=\"yes\">Categ_ory icon size:</property><pr"
  "operty name=\"use_underline\">True</property><property name=\"mnemonic_"
  "widget\">category-icon-size</property></object><packing><property name="
  "\"top_attach\">3</property><property name=\"bottom_attach\">4</property"
  "><property name=\"x_options\">GTK_FILL</property></packing></child><chi"
  "ld><object class=\"GtkComboBox\" id=\"item-icon-size\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"model\">icon-sizes</property><child><object class=\"Gtk"
  "CellRendererText\" id=\"cellrenderertext4\"/><attributes><attribute nam"
  "e=\"text\">0</attribute></attributes></child></object><packing><propert"
  "y name=\"left_attach\">1</property><property name=\"right_attach\">2</p"
  "roperty><property name=\"top_attach\">2</property><property name=\"bott"
  "om_attach\">3</property></packing></child><child><object class=\"GtkCom"
  "boBox\" id=\"category-icon-size\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">False</property><property name=\"mode"
  "l\">icon-sizes</property><child><object class=\"GtkCellRendererText\" i"
  "d=\"cellrenderertext2\"/><attributes><attribute name=\"text\">0</attrib"
  "ute></attributes></child></object><packing><property name=\"left_attach"
  "\">1</property><property name=\"right_attach\">2</property><property na"
  "me=\"top_attach\">3</property><property name=\"bottom_attach\">4</prope"
  "rty></packing></child></object></child></object></child><child type=\"l"
  "abel\"><object class=\"GtkLabel\" id=\"label5\"><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">False</property><proper"
  "ty name=\"label\" translatable=\"yes\">Appearance</property><property n"
  "ame=\"use_markup\">True</property><attributes><attribute name=\"weight\""
  " value=\"bold\"/></attributes></object></child></object><packing><prope"
  "rty name=\"expand\">True</property><property name=\"fill\">True</proper"
  "ty><property name=\"position\">1</property></packing></child><child><ob"
  "ject class=\"GtkFrame\" id=\"frame2\"><property name=\"visible\">True</"
  "property><property name=\"can_focus\">False</property><property name=\""
  "label_xalign\">0</property><property name=\"shadow_type\">none</propert"
  "y><child><object class=\"GtkAlignment\" id=\"alignment3\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">False</proper"
  "ty><property name=\"border_width\">6</property><property name=\"xalign\""
  ">0</property><property name=\"xscale\">0</property><property name=\"ysc"
  "ale\">0</property><property name=\"left_padding\">12</property><child><"
  "object class=\"GtkButton\" id=\"button-clear\"><property name=\"label\""
  " translatable=\"yes\">C_lear Custom Command History</property><property"
  " name=\"use_action_appearance\">False</property><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">True</property><propert"
  "y name=\"receives_default\">True</property><property name=\"image\">ima"
  "ge3</property><property name=\"use_underline\">True</property></object>"
  "</child></object></child><child type=\"label\"><object class=\"GtkLabel"
  "\" id=\"label7\"><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">False</property><property name=\"label\" translatable="
  "\"yes\">History</property><attributes><attribute name=\"weight\" value="
  "\"bold\"/></attributes></object></child></object><packing><property nam"
  "e=\"expand\">False</property><property name=\"fill\">True</property><pr"
  "operty name=\"position\">2</property></packing></child></object></child"
  "><child type=\"tab\"><object class=\"GtkLabel\" id=\"label1\"><property"
  " name=\"visible\">True</property><property name=\"can_focus\">False</pr"
  "operty><property name=\"label\" translatable=\"yes\">_General</property"
  "><property name=\"use_underline\">True</property></object><packing><pro"
  "perty name=\"tab_fill\">False</property></packing></child><child><objec"
  "t class=\"GtkVBox\" id=\"vbox1\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><property name=\"borde"
  "r_width\">6</property><property name=\"spacing\">6</property><child><ob"
  "ject class=\"GtkHBox\" id=\"hbox1\"><property name=\"visible\">True</pr"
  "operty><property name=\"can_focus\">False</property><property name=\"sp"
  "acing\">6</property><child><object class=\"GtkScrolledWindow\" id=\"scr"
  "olledwindow1\"><property name=\"visible\">True</property><property name"
  "=\"can_focus\">True</property><property name=\"hscrollbar_policy\">auto"
  "matic</property><property name=\"vscrollbar_policy\">automatic</propert"
  "y><property name=\"shadow_type\">etched-in</property><child><object cla"
  "ss=\"GtkTreeView\" id=\"actions-treeview\"><property name=\"visible\">T"
  "rue</property><property name=\"can_focus\">True</property><property nam"
  "e=\"model\">actions-store</property><property name=\"headers_clickable\""
  ">False</property><property name=\"rules_hint\">True</property><property"
  " name=\"enable_search\">False</property><property name=\"search_column\""
  ">0</property><child><object class=\"GtkTreeViewColumn\" id=\"treeviewco"
  "lumn1\"><property name=\"title\" translatable=\"yes\">Pattern</property"
  "><child><object class=\"GtkCellRendererText\" id=\"cellrenderertext1\"/"
  "><attributes><attribute name=\"text\">0</attribute></attributes></child"
  "></object></child></object></child></object><packing><property name=\"e"
  "xpand\">True</property><property name=\"fill\">True</property><property"
  " name=\"position\">0</property></packing></child><child><object class=\""
  "GtkAlignment\" id=\"alignment1\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><property name=\"yalig"
  "n\">0</property><property name=\"xscale\">0</property><property name=\""
  "yscale\">0</property><child><object class=\"GtkVBox\" id=\"vbox2\"><pro"
  "perty name=\"visible\">True</property><property name=\"can_focus\">Fals"
  "e</property><property name=\"spacing\">6</property><child><object class"
  "=\"GtkButton\" id=\"button-add\"><property name=\"use_action_appearance"
  "\">False</property><property name=\"visible\">True</property><property "
  "name=\"can_focus\">True</property><property name=\"receives_default\">T"
  "rue</property><property name=\"tooltip_text\" translatable=\"yes\">Add "
  "a new custom action.</property><child><object class=\"GtkImage\" id=\"i"
  "mage1\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"stock\">gtk-add</property></ob"
  "ject></child></object><packing><property name=\"expand\">True</property"
  "><property name=\"fill\">True</property><property name=\"position\">0</"
  "property></packing></child><child><object class=\"GtkButton\" id=\"butt"
  "on-remove\"><property name=\"use_action_appearance\">False</property><p"
  "roperty name=\"visible\">True</property><property name=\"can_focus\">Tr"
  "ue</property><property name=\"receives_default\">True</property><proper"
  "ty name=\"tooltip_text\" translatable=\"yes\">Remove the currently sele"
  "cted action.</property><child><object class=\"GtkImage\" id=\"image2\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"stock\">gtk-delete</property></object>"
  "</child></object><packing><property name=\"expand\">True</property><pro"
  "perty name=\"fill\">True</property><property name=\"position\">1</prope"
  "rty></packing></child></object></child></object><packing><property name"
  "=\"expand\">False</property><property name=\"fill\">True</property><pro"
  "perty name=\"position\">1</property></packing></child></object><packing"
  "><property name=\"expand\">True</property><property name=\"fill\">True<"
  "/property><property name=\"position\">0</property></packing></child><ch"
  "ild><object class=\"GtkTable\" id=\"table1\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"n_rows\">4</property><property name=\"n_columns\">2</property><p"
  "roperty name=\"column_spacing\">12</property><property name=\"row_spaci"
  "ng\">6</property><child><object class=\"GtkEntry\" id=\"command\"><prop"
  "erty name=\"visible\">True</property><property name=\"can_focus\">True<"
  "/property><property name=\"tooltip_text\" translatable=\"yes\">If the t"
  "ype is set to prefix, %s will be replaced with the string after the pat"
  "tern, %S with the complete entry text. For regular expressions you can "
  "use \\0 and \\&lt;num&gt;.</property><property name=\"invisible_char\">"
  "\342\200\242</property><property name=\"invisible_char_set\">True</prop"
  "erty><property name=\"primary_icon_activatable\">False</property><prope"
  "rty name=\"secondary_icon_activatable\">False</property><property name="
  "\"primary_icon_sensitive\">True</property><property name=\"secondary_ic"
  "on_sensitive\">True</property></object><packing><property name=\"left_a"
  "ttach\">1</property><property name=\"right_attach\">2</property><proper"
  "ty name=\"top_attach\">2</property><property name=\"bottom_attach\">3</"
  "property></packing></child><child><object class=\"GtkEntry\" id=\"patte"
  "rn\"><property name=\"visible\">True</property><property name=\"can_foc"
  "us\">True</property><property name=\"invisible_char\">\342\200\242</pro"
  "perty><property name=\"invisible_char_set\">True</property><property na"
  "me=\"primary_icon_activatable\">False</property><property name=\"second"
  "ary_icon_activatable\">False</property><property name=\"primary_icon_se"
  "nsitive\">True</property><property name=\"secondary_icon_sensitive\">Tr"
  "ue</property></object><packing><property name=\"left_attach\">1</proper"
  "ty><property name=\"right_attach\">2</property><property name=\"top_att"
  "ach\">1</property><property name=\"bottom_attach\">2</property></packin"
  "g></child><child><object class=\"GtkLabel\" id=\"label4\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">False</proper"
  "ty><property name=\"xalign\">0</property><property name=\"label\" trans"
  "latable=\"yes\">Co_mmand:</property><property name=\"use_underline\">Tr"
  "ue</property><property name=\"mnemonic_widget\">command</property></obj"
  "ect><packing><property name=\"top_attach\">2</property><property name=\""
  "bottom_attach\">3</property><property name=\"x_options\">GTK_FILL</prop"
  "erty></packing></child><child><object class=\"GtkLabel\" id=\"label3\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"xalign\">0</property><property name=\""
  "label\" translatable=\"yes\">Patte_rn:</property><property name=\"use_u"
  "nderline\">True</property><property name=\"mnemonic_widget\">pattern</p"
  "roperty></object><packing><property name=\"top_attach\">1</property><pr"
  "operty name=\"bottom_attach\">2</property><property name=\"x_options\">"
  "GTK_FILL</property></packing></child><child><object class=\"GtkLabel\" "
  "id=\"label8\"><property name=\"visible\">True</property><property name="
  "\"can_focus\">False</property><property name=\"xalign\">0</property><pr"
  "operty name=\"label\" translatable=\"yes\">_Type:</property><property n"
  "ame=\"use_underline\">True</property><property name=\"mnemonic_widget\""
  ">type</property></object><packing><property name=\"x_options\">GTK_FILL"
  "</property></packing></child><child><object class=\"GtkAlignment\" id=\""
  "alignment4\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"xalign\">0</property><prop"
  "erty name=\"xscale\">0</property><child><object class=\"GtkComboBox\" i"
  "d=\"type\"><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">False</property><property name=\"model\">action-types</prope"
  "rty><child><object class=\"GtkCellRendererText\" id=\"cellrenderertext3"
  "\"/><attributes><attribute name=\"text\">0</attribute></attributes></ch"
  "ild></object></child></object><packing><property name=\"left_attach\">1"
  "</property><property name=\"right_attach\">2</property></packing></chil"
  "d><child><object class=\"GtkCheckButton\" id=\"save\"><property name=\""
  "label\" translatable=\"yes\">_Save match in command history</property><"
  "property name=\"use_action_appearance\">False</property><property name="
  "\"visible\">True</property><property name=\"can_focus\">True</property>"
  "<property name=\"receives_default\">False</property><property name=\"us"
  "e_underline\">True</property><property name=\"draw_indicator\">True</pr"
  "operty></object><packing><property name=\"left_attach\">1</property><pr"
  "operty name=\"right_attach\">2</property><property name=\"top_attach\">"
  "3</property><property name=\"bottom_attach\">4</property></packing></ch"
  "ild><child><placeholder/></child></object><packing><property name=\"exp"
  "and\">False</property><property name=\"fill\">True</property><property "
  "name=\"position\">1</property></packing></child></object><packing><prop"
  "erty name=\"position\">1</property></packing></child><child type=\"tab\""
  "><object class=\"GtkLabel\" id=\"label2\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"label\" translatable=\"yes\">Custom _Actions</property><property na"
  "me=\"use_underline\">True</property></object><packing><property name=\""
  "position\">1</property><property name=\"tab_fill\">False</property></pa"
  "cking></child></object><packing><property name=\"expand\">True</propert"
  "y><property name=\"fill\">True</property><property name=\"position\">1<"
  "/property></packing></child></object></child><action-widgets><action-wi"
  "dget response=\"0\">button-close</action-widget><action-widget response"
  "=\"-11\">button-help</action-widget></action-widgets></object></interfa"
  "ce>"
};

static const unsigned appfinder_preferences_ui_length = 21594u;

